@@ -1,6 +1,6 @@
 ##
 ## intsurv: Integrative Survival Models
-## Copyright (C) 2017-2019  Wenjie Wang <wjwang.stat@gmail.com>
+## Copyright (C) 2017-2021  Wenjie Wang <wang@wwenjie.org>
 ##
 ## This file is part of the R package intsurv.
 ##
@@ -21,7 +21,7 @@
 ##' For right-censored data, fit a regularized Cox cure rate model through
 ##' elastic-net penalty following Masud et al. (2018), and Zou and Hastie
 ##' (2005).  For right-censored data with uncertain event status,
-##' fit the regularized Cox cure model proposed by Wang et al. (2019+).  Without
+##' fit the regularized Cox cure model proposed by Wang et al. (2020).  Without
 ##' regularization, the model reduces to the regular Cox cure rate model (Kuk
 ##' and Chen, 1992; Sy and Taylor, 2000)
 ##'
@@ -30,100 +30,38 @@
 ##' net penalty is developed based on cyclic coordinate descent and
 ##' majorization-minimization (MM) algorithm.
 ##'
-##' @usage
 ##'
-##' cox_cure_net(surv_formula, cure_formula, time, event,
-##'              data, subset, contrasts = NULL,
-##'              surv_lambda, surv_alpha = 1, surv_nlambda = 10,
-##'              surv_lambda_min_ratio = 1e-1, surv_l1_penalty_factor,
-##'              cure_lambda, cure_alpha = 1, cure_nlambda = 10,
-##'              cure_lambda_min_ratio = 1e-1, cure_l1_penalty_factor,
-##'              surv_start, cure_start,
-##'              surv_standardize = TRUE, cure_standardize = TRUE,
-##'              em_max_iter = 200, em_rel_tol = 1e-5,
-##'              surv_max_iter = 10, surv_rel_tol = 1e-5,
-##'              cure_max_iter = 10, cure_rel_tol = 1e-5,
-##'              tail_completion = c("zero", "exp", "zero-tau"),
-##'              tail_tau = NULL, pmin = 1e-5, early_stop = TRUE,
-##'              verbose = FALSE, ...)
+##' @inheritParams cox_cure
 ##'
-##' @param surv_formula A formula object starting with \code{~} for the model
-##'     formula in survival model part.  For Cox model, no intercept term is
-##'     included even if an intercept is specified or implied in the model
-##'     formula.  A model formula with an intercept term only is not allowed.
-##' @param cure_formula A formula object starting with \code{~} for the model
-##'     formula in cure rate model part.  For logistic model, an intercept term
-##'     is included by default and can be excluded by adding \code{+ 0} or
-##'     \code{- 1} to the model formula.
-##' @param time A numeric vector for the observed survival times.
-##' @param event A numeric vector for the event indicators.  \code{NA}'s are
-##'     allowed and represent uncertain event status.
-##' @param data An optional data frame, list, or environment that contains the
-##'     covariates and response variables (\code{time} and \code{event}),
-##'     included in the model. If they are not found in data, the variables are
-##'     taken from the environment of the specified formula, usually the
-##'     environment from which this function is called.
-##' @param subset An optional logical vector specifying a subset of observations
-##'     to be used in the fitting process.
-##' @param contrasts An optional list, whose entries are values (numeric
-##'     matrices or character strings naming functions) to be used as
-##'     replacement values for the contrasts replacement function and whose
-##'     names are the names of columns of data containing factors.  See
-##'     \code{contrasts.arg} of \code{\link[stats]{model.matrix.default}} for
-##'     details.
-##' @param surv_lambda A numeric vector consists of non-negative values
-##'     representing the tuning parameter sequence for the survival model part.
-##' @param surv_alpha A number between 0 and 1 for tuning the elastic net
-##'     penalty for the survival model part.  If it is one, the elastic penalty
-##'     will reduce to the well-known lasso penalty.  If it is zero, the ridge
-##'     penalty will be used.
-##' @param surv_nlambda A positive number specifying the number of
-##'     \code{surv_lambda} if \code{surv_lambda} is not specified.  The default
-##'     value is 10.
-##' @param surv_lambda_min_ratio The ratio of the minimum \code{surv_lambda} to
-##'     the large enough \code{surv_lambda} that produces all-zero estimates on
-##'     log scale.  The default value is \code{1e-1}.
-##' @param surv_l1_penalty_factor A numeric vector that consists of positive
-##'     numbers for penalty factors (or weights) on L1-norm for the coefficient
-##'     estimate vector in the survival model part.  The penalty is applied to
-##'     the coefficient estimate divided by the specified weights.  The
-##'     specified weights are re-scaled internally so that their summation
-##'     equals the length of coefficients.  If it is left unspecified, the
-##'     weights are all set to be one.
-##' @param cure_lambda A numeric vector consists of non-negative values
-##'     representing the tuning parameter sequence for the cure model part.
-##' @param cure_alpha A number between 0 and 1 for tuning the elastic net
-##'     penalty for the cure model part.  If it is one, the elastic penalty will
-##'     reduce to the well-known lasso penalty.  If it is zero, the ridge
-##'     penalty will be used.
-##' @param cure_nlambda A positive number specifying the number of
-##'     \code{cure_lambda} if \code{cure_lambda} is not specified.  The default
-##'     value is 10.
-##' @param cure_lambda_min_ratio The ratio of the minimum \code{cure_lambda} to
-##'     the large enough \code{cure_lambda} that produces all-zero estimates on
-##'     log scale.  The default value is \code{1e-1}.
-##' @param cure_l1_penalty_factor A numeric vector that consists of positive
-##'     numbers for penalty factors (or weights) on L1-norm for the coefficient
-##'     estimate vector in the cure model part.  The penalty is applied to the
-##'     coefficient estimate divided by the specified weights.  The specified
-##'     weights are re-scaled internally so that their summation equals the
-##'     length of coefficients.  If it is left unspecified, the weights are all
-##'     set to be one.
-##' @param surv_start An optional numeric vector representing the starting
-##'     values for the Cox model component.  If not specified, the starting
-##'     values will be obtained from fitting a regular Cox model to events only.
-##' @param cure_start An optional numeric vector representing the starting
-##'     values for the logistic model component.  If not specified, the starting
-##'     values will be obtained from fitting a regular logistic model to the
-##'     non-missing event indicators.
-##' @param surv_standardize A logical value specifying whether to standardize
-##'     the covariates for the survival model part.  If \code{FALSE}, the
-##'     covariates will be standardized internally to have mean zero and
-##'     standard deviation one.
-##' @param cure_standardize A logical value specifying whether to standardize
-##'     the covariates for the cure rate model part.  If \code{TRUE} (by
-##'     default), the covariates will be standardized internally to have mean
-##'     zero and standard deviation one.
+##' @param surv_lambda,cure_lambda A numeric vector consists of nonnegative
+##'     values representing the tuning parameter sequence for the survival model
+##'     part or the incidence model part.
+##' @param surv_alpha,cure_alpha A number between 0 and 1 for tuning the elastic
+##'     net penalty for the survival model part or the incidence model part.  If
+##'     it is one, the elastic penalty will reduce to the well-known lasso
+##'     penalty.  If it is zero, the ridge penalty will be used.
+##' @param surv_nlambda,cure_nlambda A positive number specifying the number of
+##'     \code{surv_lambda} or \code{cure_lambda} if \code{surv_lambda} or
+##'     \code{cure_lambda} is not specified, respectively.  The default value is
+##'     10.
+##' @param surv_lambda_min_ratio,cure_lambda_min_ratio The ratio of the minimum
+##'     \code{surv_lambda} (or \code{cure_lambda}) to the large enough
+##'     \code{surv_lambda} (or code{cure_lambda}) that produces all-zero
+##'     estimates on log scale.  The default value is \code{1e-1}.
+##' @param surv_l1_penalty_factor,cure_l1_penalty_factor A numeric vector that
+##'     consists of nonnegative penalty factors (or weights) on L1-norm for the
+##'     coefficient estimate vector in the survival model part or the incidence
+##'     model part.  The penalty is applied to the coefficient estimate divided
+##'     by the specified weights.  The specified weights are re-scaled
+##'     internally so that their summation equals the length of coefficients.
+##'     If \code{NULL} is specified, the weights are all set to be one.
+##' @param cv_nfolds An non-negative integer specifying number of folds in
+##'     cross-validation (CV).  The default value is \code{0} and the CV
+##'     procedure is not enabled.
+##' @param surv_standardize,cure_standardize A logical value specifying whether
+##'     to standardize the covariates for the survival model part or the
+##'     incidence model part.  If \code{FALSE}, the covariates will be
+##'     standardized internally to have mean zero and standard deviation one.
 ##' @param em_max_iter A positive integer specifying the maximum iteration
 ##'     number of the EM algorithm.  The default value is \code{200}.
 ##' @param em_rel_tol A positive number specifying the tolerance that determines
@@ -132,48 +70,10 @@
 ##'     relative change between estimates from two consecutive iterations, which
 ##'     is measured by ratio of the L1-norm of their difference to the sum of
 ##'     their L1-norm.  The default value is \code{1e-5}.
-##' @param surv_max_iter A positive integer specifying the maximum iteration
-##'     number of the M-step routine related to the survival model component.
-##'     The default value is \code{10} to encourage faster convergence.
-##' @param surv_rel_tol A positive number specifying the tolerance that
-##'     determines the convergence of the M-step related to the survival model
-##'     component in terms of the convergence of the covariate coefficient
-##'     estimates.  The tolerance is compared with the relative change between
-##'     estimates from two consecutive iterations, which is measured by ratio of
-##'     the L1-norm of their difference to the sum of their L1-norm.  The
-##'     default value is \code{1e-5}.
-##' @param cure_max_iter A positive integer specifying the maximum iteration
-##'     number of the M-step routine related to the cure rate model component.
-##'     The default value is \code{10} to encourage faster convergence.
-##' @param cure_rel_tol A positive number specifying the tolerance that
-##'     determines the convergence of the M-step related to the cure rate model
-##'     component in terms of the convergence of the covariate coefficient
-##'     estimates.  The tolerance is compared with the relative change between
-##'     estimates from two consecutive iterations, which is measured by ratio of
-##'     the L1-norm of their difference to the sum of their L1-norm.  The
-##'     default value is \code{1e-5}.
-##' @param tail_completion A character string specifying the tail completion
-##'     method for conditional survival function.  The available methods are
-##'     \code{"zero"} for zero-tail completion after the largest event times (Sy
-##'     and Taylor, 2000), \code{"exp"} for exponential-tail completion (Peng,
-##'     2003), and \code{"zero-tau"} for zero-tail completion after a specified
-##'     \code{tail_tau}.  The default method is the zero-tail completion
-##'     proposed by Sy and Taylor (2000).
-##' @param tail_tau A numeric number specifying the time of zero-tail
-##'     completion.  It will be used only if \code{tail_completion =
-##'     "zero-tau"}.  A reasonable choice must be a time point between the
-##'     largest event time and the largest survival time.
-##' @param pmin A numeric number specifying the minimum value of probabilities
-##'     for sake of numerical stability.  The default value is \code{1e-5}.
-##' @param early_stop A logical value specifying whether to stop the iteration
-##'     once the negative log-likelihood unexpectedly increases, which may
-##'     suggest convergence on likelihood, or indicate numerical issues or
-##'     implementation bugs.  The default value is \code{TRUE}.
-##' @param verbose A logical value.  If \code{TRUE}, a verbose information will
-##'     be given along iterations for tracing the convergence.  The default
-##'     value is \code{FALSE}.
-##' @param ... Other arguments for future usage.  A warning will be thrown if
-##'     any invalid argument is specified.
+##' @param surv_max_iter,cure_max_iter A positive integer specifying the maximum
+##'     iteration number of the M-step routine related to the survival model
+##'     component or the incidence model component.  The default value is
+##'     \code{10} to encourage faster convergence.
 ##'
 ##' @return
 ##'
@@ -198,9 +98,9 @@
 ##' Sy, J. P., & Taylor, J. M. (2000). Estimation in a Cox proportional hazards
 ##' cure model. \emph{Biometrics}, 56(1), 227--236.
 ##'
-##' Wang, W., Chen, K., Luo, C., & Yan, J. (2019+). Cox Cure Model with
-##' Uncertain Event Status with application to a Suicide Risk
-##' Study. \emph{Working in Progress}.
+##' Wang, W., Luo, C., Aseltine, R. H., Wang, F., Yan, J., & Chen, K. (2020).
+##' Suicide Risk Modeling with Uncertain Diagnostic Records. \emph{arXiv
+##' preprint arXiv:2009.02597}.
 ##'
 ##' Zou, H., & Hastie, T. (2005). Regularization and variable selection via the
 ##' elastic net. \emph{Journal of the Royal Statistical Society}: Series B
@@ -213,20 +113,25 @@
 ##' @example inst/examples/cox_cure_net.R
 ##' @export
 cox_cure_net <-
-    function(surv_formula, cure_formula, time, event,
-             data, subset, contrasts = NULL,
-             surv_lambda,
+    function(surv_formula, cure_formula,
+             time, event,
+             data, subset,
+             contrasts = NULL,
+             surv_lambda = NULL,
              surv_alpha = 1,
              surv_nlambda = 10,
              surv_lambda_min_ratio = 1e-1,
-             surv_l1_penalty_factor,
-             cure_lambda,
+             surv_l1_penalty_factor = NULL,
+             cure_lambda = NULL,
              cure_alpha = 1,
              cure_nlambda = 10,
              cure_lambda_min_ratio = 1e-1,
-             cure_l1_penalty_factor,
-             surv_start,
-             cure_start,
+             cure_l1_penalty_factor = NULL,
+             cv_nfolds = 0,
+             surv_start = NULL,
+             cure_start = NULL,
+             surv_offset = NULL,
+             cure_offset = NULL,
              surv_standardize = TRUE,
              cure_standardize = TRUE,
              em_max_iter = 200,
@@ -265,6 +170,8 @@ cox_cure_net <-
     obs_event <- model_list$surv$obs_event
     surv_x <- model_list$surv$xMat
     cure_x <- model_list$cure$xMat
+    surv_offset <- model_list$surv$offset
+    cure_offset <- model_list$surv$offset
 
     ## cox model does not have an intercept
     surv_is_intercept <- colnames(surv_x) == "(Intercept)"
@@ -299,16 +206,27 @@ cox_cure_net <-
         stop("No event can be found.")
     }
     ## starting values
-    if (missing(surv_start)) {
+    if (is.null(surv_start)) {
         surv_start <- 0
     } else if (length(surv_start) != surv_x) {
         stop("The length of 'surv_start' is inappropriate.")
     }
-    if (missing(cure_start)) {
+    if (is.null(cure_start)) {
         cure_start <- 0
     } else if (length(cure_start) != cure_x + as.integer(cure_intercept)) {
         stop("The length of 'cure_start' is inappropriate.")
     }
+    ## offset terms
+    if (is.null(surv_offset)) {
+        surv_offset <- rep(0, nrow(surv_x))
+    } else if (length(surv_offset) != nrow(surv_x)) {
+        stop("The length of 'surv_offset' is inappropriate.")
+    }
+    if (is.null(cure_offset)) {
+        cure_offset <- rep(0, nrow(cure_x))
+    } else if (length(cure_offset) != nrow(cure_x)) {
+        stop("The length of 'cure_start' is inappropriate.")
+    }
     ## on tail completion
     all_tails <- c("zero", "exp", "zero-tau")
     tail_completion <- match(match.arg(tail_completion, all_tails),
@@ -329,22 +247,22 @@ cox_cure_net <-
     }
 
     ## lambda sequence
-    if (missing(surv_lambda)) {
+    if (is.null(surv_lambda)) {
         surv_lambda <- 0
     } else {
         surv_nlambda <- 1
     }
-    if (missing(cure_lambda)) {
+    if (is.null(cure_lambda)) {
         cure_lambda <- 0
     } else {
         cure_nlambda <- 1
     }
 
     ## penalty factor
-    if (missing(surv_l1_penalty_factor)) {
+    if (is.null(surv_l1_penalty_factor)) {
         surv_l1_penalty_factor <- 0
     }
-    if (missing(cure_l1_penalty_factor)) {
+    if (is.null(cure_l1_penalty_factor)) {
         cure_l1_penalty_factor <- 0
     }
 
@@ -366,8 +284,11 @@ cox_cure_net <-
             cure_nlambda = cure_nlambda,
             cure_lambda_min_ratio = cure_lambda_min_ratio,
             cure_l1_penalty_factor = cure_l1_penalty_factor,
+            cv_nfolds = cv_nfolds,
             cox_start = surv_start,
             cure_start = cure_start,
+            cox_offset = surv_offset,
+            cure_offset = cure_offset,
             cox_standardize = surv_standardize,
             cure_standardize = cure_standardize,
             em_max_iter = em_max_iter,
@@ -400,8 +321,11 @@ cox_cure_net <-
             cure_nlambda = cure_nlambda,
             cure_lambda_min_ratio = cure_lambda_min_ratio,
             cure_l1_penalty_factor = cure_l1_penalty_factor,
+            cv_nfolds = cv_nfolds,
             cox_start = surv_start,
             cure_start = cure_start,
+            cox_offset = surv_offset,
+            cure_offset = cure_offset,
             cox_standardize = surv_standardize,
             cure_standardize = cure_standardize,
             em_max_iter = em_max_iter,
@@ -449,22 +373,6 @@ cox_cure_net <-
 
 ##' @rdname cox_cure_net
 ##'
-##' @usage
-##'
-##' cox_cure_net.fit(surv_x, cure_x, time, event, cure_intercept = TRUE,
-##'                  surv_lambda, surv_alpha = 1, surv_nlambda = 10,
-##'                  surv_lambda_min_ratio = 1e-1, surv_l1_penalty_factor,
-##'                  cure_lambda, cure_alpha = 1, cure_nlambda = 10,
-##'                  cure_lambda_min_ratio = 1e-1, cure_l1_penalty_factor,
-##'                  surv_start, cure_start,
-##'                  surv_standardize = TRUE, cure_standardize = TRUE,
-##'                  em_max_iter = 200, em_rel_tol = 1e-5,
-##'                  surv_max_iter = 10, surv_rel_tol = 1e-5,
-##'                  cure_max_iter = 10, cure_rel_tol = 1e-5,
-##'                  tail_completion = c("zero", "exp", "zero-tau"),
-##'                  tail_tau = NULL, pmin = 1e-5, early_stop = TRUE,
-##'                  verbose = FALSE, ...)
-##'
 ##' @param surv_x A numeric matrix for the design matrix of the survival model
 ##'     component.
 ##' @param cure_x A numeric matrix for the design matrix of the cure rate model
@@ -480,18 +388,21 @@ cox_cure_net <-
 cox_cure_net.fit <-
     function(surv_x, cure_x, time, event,
              cure_intercept = TRUE,
-             surv_lambda,
+             surv_lambda = NULL,
              surv_alpha = 1,
              surv_nlambda = 10,
              surv_lambda_min_ratio = 1e-1,
-             surv_l1_penalty_factor,
-             cure_lambda,
+             surv_l1_penalty_factor = NULL,
+             cure_lambda = NULL,
              cure_alpha = 1,
              cure_nlambda = 10,
              cure_lambda_min_ratio = 1e-1,
-             cure_l1_penalty_factor,
-             surv_start,
-             cure_start,
+             cure_l1_penalty_factor = NULL,
+             cv_nfolds = 0,
+             surv_start = NULL,
+             cure_start = NULL,
+             surv_offset = NULL,
+             cure_offset = NULL,
              surv_standardize = TRUE,
              cure_standardize = TRUE,
              em_max_iter = 200,
@@ -522,16 +433,27 @@ cox_cure_net.fit <-
         stop("No event can be found.")
     }
     ## starting values
-    if (missing(surv_start)) {
+    if (is.null(surv_start)) {
         surv_start <- 0
     } else if (length(surv_start) != surv_x) {
         stop("The length of 'surv_start' is inappropriate.")
     }
-    if (missing(cure_start)) {
+    if (is.null(cure_start)) {
         cure_start <- 0
     } else if (length(cure_start) != cure_x + as.integer(cure_intercept)) {
         stop("The length of 'cure_start' is inappropriate.")
     }
+    ## offset terms
+    if (is.null(surv_offset)) {
+        surv_offset <- rep(0, nrow(surv_x))
+    } else if (length(surv_offset) != nrow(surv_x)) {
+        stop("The length of 'surv_offset' is inappropriate.")
+    }
+    if (is.null(cure_offset)) {
+        cure_offset <- rep(0, nrow(cure_x))
+    } else if (length(cure_offset) != nrow(cure_x)) {
+        stop("The length of 'cure_start' is inappropriate.")
+    }
     ## on tail completion
     all_tails <- c("zero", "exp", "zero-tau")
     tail_completion <- match(match.arg(tail_completion, all_tails),
@@ -552,22 +474,22 @@ cox_cure_net.fit <-
     }
 
     ## lambda sequence
-    if (missing(surv_lambda)) {
+    if (is.null(surv_lambda)) {
         surv_lambda <- 0
     } else {
         surv_nlambda <- 1
     }
-    if (missing(cure_lambda)) {
+    if (is.null(cure_lambda)) {
         cure_lambda <- 0
     } else {
         cure_nlambda <- 1
     }
 
     ## penalty factor
-    if (missing(surv_l1_penalty_factor)) {
+    if (is.null(surv_l1_penalty_factor)) {
         surv_l1_penalty_factor <- 0
     }
-    if (missing(cure_l1_penalty_factor)) {
+    if (is.null(cure_l1_penalty_factor)) {
         cure_l1_penalty_factor <- 0
     }
 
@@ -589,8 +511,11 @@ cox_cure_net.fit <-
             cure_nlambda = cure_nlambda,
             cure_lambda_min_ratio = cure_lambda_min_ratio,
             cure_l1_penalty_factor = cure_l1_penalty_factor,
+            cv_nfolds = cv_nfolds,
             cox_start = surv_start,
             cure_start = cure_start,
+            cox_offset = surv_offset,
+            cure_offset = cure_offset,
             cox_standardize = surv_standardize,
             cure_standardize = cure_standardize,
             em_max_iter = em_max_iter,
@@ -623,8 +548,11 @@ cox_cure_net.fit <-
             cure_nlambda = cure_nlambda,
             cure_lambda_min_ratio = cure_lambda_min_ratio,
             cure_l1_penalty_factor = cure_l1_penalty_factor,
+            cv_nfolds = cv_nfolds,
             cox_start = surv_start,
             cure_start = cure_start,
+            cox_offset = surv_offset,
+            cure_offset = cure_offset,
             cox_standardize = surv_standardize,
             cure_standardize = cure_standardize,
             em_max_iter = em_max_iter,
@@ -656,7 +584,7 @@ cox_cure_net.fit <-
         colnames(out$cure_coef) <-
             if (cure_intercept) {
                 c("(Intercept)",
-                  paste0("z", seq_len(ncol(out$cure_coef[- 1L]))))
+                  paste0("z", seq_len(ncol(out$cure_coef) - 1L)))
             } else {
                 paste0("z", seq_len(ncol(out$cure_coef)))
             }
