@@ -1,6 +1,6 @@
 //
 // intsurv: Integrative Survival Models
-// Copyright (C) 2017-2019  Wenjie Wang <wjwang.stat@gmail.com>
+// Copyright (C) 2017-2021  Wenjie Wang <wang@wwenjie.org>
 //
 // This file is part of the R package intsurv.
 //
@@ -31,17 +31,16 @@ Rcpp::List coxph_cure_uncer(
     const unsigned int& bootstrap = 0,
     const arma::vec& cox_start = 0,
     const arma::vec& cure_start = 0,
+    const arma::vec& cox_offset = 0,
+    const arma::vec& cure_offset = 0,
+    const bool cox_standardize = true,
+    const bool cure_standardize = true,
     const unsigned int& em_max_iter = 300,
     const double& em_rel_tol = 1e-5,
     const unsigned int& cox_mstep_max_iter = 100,
     const double& cox_mstep_rel_tol = 1e-5,
     const unsigned int& cure_mstep_max_iter = 100,
     const double& cure_mstep_rel_tol = 1e-5,
-    const bool cox_standardize = true,
-    const bool cure_standardize = true,
-    const bool& spline_start = false,
-    const unsigned int& iSpline_num_knots = 3,
-    const unsigned int& iSpline_degree = 2,
     const unsigned int& tail_completion = 1,
     double tail_tau = -1,
     const double& pmin = 1e-5,
@@ -52,14 +51,14 @@ Rcpp::List coxph_cure_uncer(
     // define object
     Intsurv::CoxphCureUncer obj {
         time, event, cox_x, cure_x, cure_intercept,
-        cox_standardize, cure_standardize
+        cox_standardize, cure_standardize,
+        cox_offset, cure_offset
     };
     // model-fitting
     obj.fit(cox_start, cure_start,
             em_max_iter, em_rel_tol,
             cox_mstep_max_iter, cox_mstep_rel_tol,
             cure_mstep_max_iter, cure_mstep_rel_tol,
-            spline_start, iSpline_num_knots, iSpline_degree,
             tail_completion, tail_tau,
             pmin, early_stop, verbose
         );
@@ -91,13 +90,14 @@ Rcpp::List coxph_cure_uncer(
                 cure_x.rows(boot_ind),
                 cure_intercept,
                 cox_standardize,
-                cure_standardize
+                cure_standardize,
+                cox_offset.elem(boot_ind),
+                cure_offset.elem(boot_ind)
             };
             boot_obj.fit(cox_start, cure_start,
                          em_max_iter, em_rel_tol,
                          cox_mstep_max_iter, cox_mstep_rel_tol,
                          cure_mstep_max_iter, cure_mstep_rel_tol,
-                         spline_start, iSpline_num_knots, iSpline_degree,
                          tail_completion, tail_tau,
                          pmin, early_stop, 0);
             boot_cox_coef_mat.col(i) = boot_obj.cox_coef;
@@ -131,6 +131,7 @@ Rcpp::List coxph_cure_uncer(
             Rcpp::Named("coef_df") = obj.coef_df,
             Rcpp::Named("negLogL") = obj.negLogL,
             Rcpp::Named("c_index") = obj.c_index,
+            Rcpp::Named("aic") = obj.aic,
             Rcpp::Named("bic1") = obj.bic1,
             Rcpp::Named("bic2") = obj.bic2
             ),
@@ -161,19 +162,19 @@ Rcpp::List coxph_cure_uncer_reg(
     const double& cure_l1_lambda = 0,
     const double& cure_l2_lambda = 0,
     const arma::vec& cure_l1_penalty_factor = 0,
+    const unsigned long cv_nfolds = 0,
     const arma::vec& cox_start = 0,
     const arma::vec& cure_start = 0,
+    const arma::vec& cox_offset = 0,
+    const arma::vec& cure_offset = 0,
+    const bool cox_standardize = true,
+    const bool cure_standardize = true,
     const unsigned int& em_max_iter = 500,
     const double& em_rel_tol = 1e-4,
     const unsigned int& cox_mstep_max_iter = 200,
     const double& cox_mstep_rel_tol = 1e-4,
     const unsigned int& cure_mstep_max_iter = 200,
     const double& cure_mstep_rel_tol = 1e-4,
-    const bool cox_standardize = true,
-    const bool cure_standardize = true,
-    const bool& spline_start = false,
-    const unsigned int& iSpline_num_knots = 3,
-    const unsigned int& iSpline_degree = 2,
     const unsigned int& tail_completion = 1,
     double tail_tau = -1,
     const double& pmin = 1e-5,
@@ -184,7 +185,8 @@ Rcpp::List coxph_cure_uncer_reg(
     // define object
     Intsurv::CoxphCureUncer obj {
         time, event, cox_x, cure_x, cure_intercept,
-        cox_standardize, cure_standardize
+        cox_standardize, cure_standardize,
+        cox_offset, cure_offset
     };
     // model-fitting
     obj.regularized_fit(
@@ -194,10 +196,29 @@ Rcpp::List coxph_cure_uncer_reg(
         cox_start, cure_start, em_max_iter, em_rel_tol,
         cox_mstep_max_iter, cox_mstep_rel_tol,
         cure_mstep_max_iter, cure_mstep_rel_tol,
-        spline_start, iSpline_num_knots, iSpline_degree,
         tail_completion, tail_tau,
         pmin, early_stop, verbose
         );
+    // cross-validation
+    arma::vec cv_vec;
+    if (cv_nfolds > 1) {
+        cv_vec = Intsurv::cv_coxph_cure_uncer_reg(
+            time, event, cox_x, cure_x, cure_intercept,
+            cv_nfolds,
+            cox_l1_lambda, cox_l2_lambda,
+            cox_l1_penalty_factor,
+            cure_l1_lambda, cure_l2_lambda,
+            cure_l1_penalty_factor,
+            cox_start, cure_start,
+            cox_offset, cure_offset,
+            cox_standardize, cure_standardize,
+            em_max_iter, em_rel_tol,
+            cox_mstep_max_iter, cox_mstep_rel_tol,
+            cure_mstep_max_iter, cure_mstep_rel_tol,
+            tail_completion, tail_tau,
+            pmin, early_stop, verbose
+            );
+    }
     // return results in a list
     return Rcpp::List::create(
         Rcpp::Named("cox_coef") = Intsurv::arma2rvec(obj.cox_coef),
@@ -229,8 +250,10 @@ Rcpp::List coxph_cure_uncer_reg(
             Rcpp::Named("coef_df") = obj.coef_df,
             Rcpp::Named("negLogL") = obj.negLogL,
             Rcpp::Named("c_index") = obj.c_index,
+            Rcpp::Named("aic") = obj.aic,
             Rcpp::Named("bic1") = obj.bic1,
-            Rcpp::Named("bic2") = obj.bic2
+            Rcpp::Named("bic2") = obj.bic2,
+            Rcpp::Named("cv_logL") = Intsurv::arma2rvec(cv_vec)
             ),
         Rcpp::Named("penalty") = Rcpp::List::create(
             Rcpp::Named("cox_l1_lambda_max") = obj.cox_l1_lambda_max,
@@ -272,19 +295,19 @@ Rcpp::List coxph_cure_uncer_vs(
     const unsigned int& cure_nlambda = 1,
     const double& cure_lambda_min_ratio = 1e-4,
     const arma::vec& cure_l1_penalty_factor = 0,
+    const unsigned long cv_nfolds = 0,
     const arma::vec& cox_start = 0,
     const arma::vec& cure_start = 0,
+    const arma::vec& cox_offset = 0,
+    const arma::vec& cure_offset = 0,
+    const bool cox_standardize = true,
+    const bool cure_standardize = true,
     const unsigned int& em_max_iter = 500,
     const double& em_rel_tol = 1e-4,
     const unsigned int& cox_mstep_max_iter = 200,
     const double& cox_mstep_rel_tol = 1e-4,
     const unsigned int& cure_mstep_max_iter = 200,
     const double& cure_mstep_rel_tol = 1e-4,
-    const bool cox_standardize = true,
-    const bool cure_standardize = true,
-    const bool& spline_start = false,
-    const unsigned int& iSpline_num_knots = 3,
-    const unsigned int& iSpline_degree = 2,
     const unsigned int& tail_completion = 1,
     double tail_tau = -1,
     const double& pmin = 1e-5,
@@ -295,7 +318,8 @@ Rcpp::List coxph_cure_uncer_vs(
     // define object
     Intsurv::CoxphCureUncer obj {
         time, event, cox_x, cure_x, cure_intercept,
-        cox_standardize, cure_standardize
+        cox_standardize, cure_standardize,
+        cox_offset, cure_offset
     };
     // get the maximum lambdas by setting em_max_iter = 0
     obj.regularized_fit(
@@ -303,7 +327,6 @@ Rcpp::List coxph_cure_uncer_vs(
         cox_start, cure_start, 0, em_rel_tol,
         cox_mstep_max_iter, cox_mstep_rel_tol,
         cure_mstep_max_iter, cure_mstep_rel_tol,
-        spline_start, iSpline_num_knots, iSpline_degree,
         tail_completion, tail_tau,
         pmin, early_stop, 0
         );
@@ -353,9 +376,10 @@ Rcpp::List coxph_cure_uncer_vs(
     arma::mat cure_coef_mat { arma::zeros(cure_p, n_lambda) };
     arma::mat cox_en_coef_mat { arma::zeros(cox_p, n_lambda) };
     arma::mat cure_en_coef_mat { arma::zeros(cure_p, n_lambda) };
-    arma::vec bic1 { arma::zeros(n_lambda) }, bic2 { bic1 };
+    arma::vec bic1 { arma::zeros(n_lambda) }, bic2 { bic1 }, aic { bic1 };
     arma::vec coef_df { bic1 }, negLogL { bic1 };
     arma::mat lambda_mat { arma::zeros(n_lambda, 4) };
+    arma::vec cv_loglik { arma::zeros(n_lambda) };
 
     // warm starts
     arma::vec cox_warm_start0 { cox_start };
@@ -383,10 +407,29 @@ Rcpp::List coxph_cure_uncer_vs(
                 em_max_iter, em_rel_tol,
                 cox_mstep_max_iter, cox_mstep_rel_tol,
                 cure_mstep_max_iter, cure_mstep_rel_tol,
-                spline_start, iSpline_num_knots, iSpline_degree,
                 tail_completion, tail_tau,
                 pmin, early_stop, verbose
                 );
+            // cross-validation
+            arma::vec cv_vec;
+            if (cv_nfolds > 1) {
+                cv_vec = Intsurv::cv_coxph_cure_uncer_reg(
+                    time, event, cox_x, cure_x,
+                    cure_intercept, cv_nfolds,
+                    cox_l1_lambda, cox_l2_lambda,
+                    cox_l1_penalty_factor,
+                    cure_l1_lambda, cure_l2_lambda,
+                    cure_l1_penalty_factor,
+                    cox_warm_start, cure_warm_start,
+                    cox_offset, cure_offset,
+                    cox_standardize, cure_standardize,
+                    em_max_iter, em_rel_tol,
+                    cox_mstep_max_iter, cox_mstep_rel_tol,
+                    cure_mstep_max_iter, cure_mstep_rel_tol,
+                    tail_completion, tail_tau,
+                    pmin, early_stop, 0
+                    );
+            }
             // update starting value
             cox_warm_start = obj.cox_coef;
             cure_warm_start = obj.cure_coef;
@@ -400,6 +443,7 @@ Rcpp::List coxph_cure_uncer_vs(
             cure_coef_mat.col(iter) = obj.cure_coef;
             cox_en_coef_mat.col(iter) = obj.cox_en_coef;
             cure_en_coef_mat.col(iter) = obj.cure_en_coef;
+            aic(iter) = obj.aic;
             bic1(iter) = obj.bic1;
             bic2(iter) = obj.bic2;
             coef_df(iter) = obj.coef_df;
@@ -408,6 +452,7 @@ Rcpp::List coxph_cure_uncer_vs(
             lambda_mat(iter, 1) = cox_l2_lambda;
             lambda_mat(iter, 2) = cure_l1_lambda;
             lambda_mat(iter, 3) = cure_l2_lambda;
+            cv_loglik(iter) = arma::sum(cv_vec);
             // update iterators
             iter++;
         }
@@ -423,8 +468,10 @@ Rcpp::List coxph_cure_uncer_vs(
             Rcpp::Named("nEvent") = obj.nEvent,
             Rcpp::Named("coef_df") = Intsurv::arma2rvec(coef_df),
             Rcpp::Named("negLogL") = Intsurv::arma2rvec(negLogL),
+            Rcpp::Named("aic") = Intsurv::arma2rvec(aic),
             Rcpp::Named("bic1") = Intsurv::arma2rvec(bic1),
-            Rcpp::Named("bic2") = Intsurv::arma2rvec(bic2)
+            Rcpp::Named("bic2") = Intsurv::arma2rvec(bic2),
+            Rcpp::Named("cv_logL") = Intsurv::arma2rvec(cv_loglik)
             ),
         Rcpp::Named("penalty") = Rcpp::List::create(
             Rcpp::Named("lambda_mat") = lambda_mat,
