@@ -1,6 +1,6 @@
 //
 // intsurv: Integrative Survival Models
-// Copyright (C) 2017-2019  Wenjie Wang <wjwang.stat@gmail.com>
+// Copyright (C) 2017-2021  Wenjie Wang <wang@wwenjie.org>
 //
 // This file is part of the R package intsurv.
 //
@@ -28,6 +28,7 @@ Rcpp::List rcpp_logistic(
     const arma::vec& y,
     const bool& intercept = true,
     const bool& standardize = true,
+    const arma::vec& offset = 0,
     const arma::vec& start = 0,
     const unsigned int& max_iter = 300,
     const double& rel_tol = 1e-6,
@@ -39,10 +40,12 @@ Rcpp::List rcpp_logistic(
     Intsurv::LogisticReg object {
         x, y, intercept, standardize
     };
+    object.set_offset(offset);
     object.fit(start, max_iter, rel_tol, pmin, early_stop, verbose);
     return Rcpp::List::create(
         Rcpp::Named("coef") = Intsurv::arma2rvec(object.coef),
         Rcpp::Named("model") = Rcpp::List::create(
+            Rcpp::Named("fitted") = Intsurv::arma2rvec(object.prob_vec),
             Rcpp::Named("nObs") = object.nObs,
             Rcpp::Named("negLogL") = object.negLogL,
             Rcpp::Named("coef_df") = object.coef_df
@@ -58,6 +61,7 @@ Rcpp::List rcpp_firth_logistic(
     const arma::vec& y,
     const bool intercept = true,
     const bool standardize = true,
+    const arma::vec& offset = 0,
     const arma::vec start = 0,
     const unsigned int max_iter = 300,
     const double rel_tol = 1e-6
@@ -66,10 +70,12 @@ Rcpp::List rcpp_firth_logistic(
     Intsurv::LogisticReg object {
         x, y, intercept, standardize
     };
+    object.set_offset(offset);
     object.firth_fit(start, max_iter, rel_tol);
     return Rcpp::List::create(
         Rcpp::Named("coef") = Intsurv::arma2rvec(object.coef),
         Rcpp::Named("model") = Rcpp::List::create(
+            Rcpp::Named("fitted") = Intsurv::arma2rvec(object.prob_vec),
             Rcpp::Named("nObs") = object.nObs,
             Rcpp::Named("negLogL") = object.negLogL,
             Rcpp::Named("coef_df") = object.coef_df
@@ -88,14 +94,19 @@ Rcpp::List rcpp_reg_logistic1(const arma::mat& x,
                               const arma::vec& l1_penalty_factor = 0,
                               const arma::vec& start = 0,
                               const bool intercept = true,
+                              const bool standardize = true,
+                              const arma::vec& offset = 0,
                               const unsigned int max_iter = 300,
                               const double rel_tol = 1e-5,
+                              const double pmin = 1e-5,
                               const bool& early_stop = false,
                               const bool& verbose = false)
 {
-    Intsurv::LogisticReg object { x, y, intercept };
+    Intsurv::LogisticReg object { x, y, intercept, standardize };
+    object.set_offset(offset);
     object.regularized_fit(l1_lambda, l2_lambda, l1_penalty_factor,
-                           start, max_iter, rel_tol, early_stop, verbose);
+                           start, max_iter, rel_tol, pmin,
+                           early_stop, verbose);
     return Rcpp::List::create(
         Rcpp::Named("coef") = Intsurv::arma2rvec(object.coef),
         Rcpp::Named("en_coef") = Intsurv::arma2rvec(object.en_coef),
@@ -125,12 +136,15 @@ Rcpp::List rcpp_reg_logistic2(const arma::mat& x,
                               double lambda_min_ratio = 1e-4,
                               const arma::vec& penalty_factor = 0,
                               const bool intercept = true,
+                              const bool standardize = true,
+                              const arma::vec& offset = 0,
                               const unsigned int max_iter = 300,
                               const double rel_tol = 1e-5,
                               const bool& early_stop = false,
                               const bool& verbose = false)
 {
-    Intsurv::LogisticReg object { x, y, intercept };
+    Intsurv::LogisticReg object { x, y, intercept, standardize };
+    object.set_offset(offset);
     object.regularized_fit(lambda, alpha, nlambda, lambda_min_ratio,
                            penalty_factor, max_iter, rel_tol,
                            early_stop, verbose);
